/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */

#ifndef __com_sun_star_script_EventListener_idl__
#define __com_sun_star_script_EventListener_idl__

#include <com/sun/star/script/XAllListener.idl>

module com {  module sun {  module star {  module script {

struct EventListener
{
    com::sun::star::script::XAllListener AllListener;
    any Helper;
    string ListenerType;
    string AddListenerParam;
    string EventMethod;
};

}; }; }; };

#endif

/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
