/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This file incorporates work covered by the following license notice:
 *
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements. See the NOTICE file distributed
 *   with this work for additional information regarding copyright
 *   ownership. The ASF licenses this file to you under the Apache
 *   License, Version 2.0 (the "License"); you may not use this file
 *   except in compliance with the License. You may obtain a copy of
 *   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 */
#ifndef __com_sun_star_registry_RegistryValueType_idl__
#define __com_sun_star_registry_RegistryValueType_idl__



 module com {  module sun {  module star {  module registry {

/** represents all possible types of a key value.

    <p>An ASCII value and a string value will both be handled with type
    string. But interns will be handled differently. Normally the idl
    string represents a unicode string. </p>
 */
published enum RegistryValueType
{
    /** the type of the key is not defined.
     */
    NOT_DEFINED,
    /** the type of the key is long.
     */
    LONG,
    /** the type of the key is ASCII.
     */
    ASCII,
    /** the type of the key is a string.
     */
    STRING,
    /** the type of the key is binary.
     */
    BINARY,
    /** the type of the key is LONGLIST.
     */
    LONGLIST,
    /** the type of the key is an ASCIILIST.
     */
    ASCIILIST,
    /** the type of the key is a STRINGLIST.
     */
    STRINGLIST

};


}; }; }; };

#endif

/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
