/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This file incorporates work covered by the following license notice:
 *
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements. See the NOTICE file distributed
 *   with this work for additional information regarding copyright
 *   ownership. The ASF licenses this file to you under the Apache
 *   License, Version 2.0 (the "License"); you may not use this file
 *   except in compliance with the License. You may obtain a copy of
 *   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 */

#ifndef __com_sun_star_reflection_XConstantTypeDescription_idl__
#define __com_sun_star_reflection_XConstantTypeDescription_idl__

#include <com/sun/star/reflection/XTypeDescription.idl>


 module com {  module sun {  module star {  module reflection {


/** Reflects a constant.

    <p>The type class of this type is
    com::sun::star::uno::TypeClass::CONSTANT.

    <p>Constants may be contained in constants groups and modules.

    @see XModuleTypeDescription
    @see XConstantsTypeDescription

    @since OOo 1.1.2

  */
published interface XConstantTypeDescription : com::sun::star::reflection::XTypeDescription
{
    /** @return the value of the constant.

        <p>Following types are allowed for constants:
        <ul>
        <li>boolean
        <li>byte
        <li>short
        <li>unsigned short
        <li>long
        <li>unsigned long
        <li>hyper
        <li>unsigned hyper
        <li>float
        <li>double
        </ul>
      */
    any getConstantValue();

};


}; }; }; };

#endif

/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
