/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This file incorporates work covered by the following license notice:
 *
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements. See the NOTICE file distributed
 *   with this work for additional information regarding copyright
 *   ownership. The ASF licenses this file to you under the Apache
 *   License, Version 2.0 (the "License"); you may not use this file
 *   except in compliance with the License. You may obtain a copy of
 *   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 */
#ifndef __com_sun_star_container_XImplicitIDContainer_idl__
#define __com_sun_star_container_XImplicitIDContainer_idl__

#include <com/sun/star/container/XImplicitIDReplace.idl>

#include <com/sun/star/container/NoSuchElementException.idl>

#include <com/sun/star/lang/WrappedTargetException.idl>



module com {  module sun {  module star {  module container {

/** makes it possible to insert and remove elements in/from a container
    using an implicit (unique) ID.
 */
published interface XImplicitIDContainer: com::sun::star::container::XImplicitIDReplace
{
    /** adds a new object to the container and generates an implicit (unique)
        ID for this object.

        @returns
            the implicit ID for the new object.
     */
    string addWithImplicitID( [in] any aElement );

    /** removes an object from the container which is specified by
        an implicit (unique) identifier.
     */
    void removeByImplicitID( [in] string ID )
        raises( com::sun::star::container::NoSuchElementException );
};


}; }; }; };

#endif

/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
