/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This file incorporates work covered by the following license notice:
 *
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements. See the NOTICE file distributed
 *   with this work for additional information regarding copyright
 *   ownership. The ASF licenses this file to you under the Apache
 *   License, Version 2.0 (the "License"); you may not use this file
 *   except in compliance with the License. You may obtain a copy of
 *   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 */
#ifndef __com_sun_star_util_BootstrapMacroExpander_idl__
#define __com_sun_star_util_BootstrapMacroExpander_idl__

#include <com/sun/star/util/MacroExpander.idl>
#include <com/sun/star/lang/XInitialization.idl>



module com {  module sun {  module star {  module util {


/** Macro expander expanding using rtl bootstrap notation.

    For details, have a look at
    http://udk.openoffice.org/common/man/concept/micro_deployment.html#misc

    The service can be instantiated via arguments giving an ini/rc file url.
    Otherwise it will read from a uno.ini/unorc file next to the cppuhelper
    library.

    @see theMacroExpander

    @since OOo 1.1.2
*/
published service BootstrapMacroExpander
{
    service MacroExpander;
    [optional] interface com::sun::star::lang::XInitialization;
};

}; }; }; };

#endif

/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
